# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/jupyter_notebook.ipynb.

# %% auto 0
__all__ = ['is_in_notebook', 'save_notebook', 'backup_this_notebook', 'backup_all_notebooks', 'display_dfs_side_by_side',
           'show_big_dataframe', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'shutdown_current_notebook']

# %% ../nbs/jupyter_notebook.ipynb 2
import importlib
import os, sys, json, time, hashlib
from IPython.display import display, Javascript, display_html, Markdown
import nbformat
from nbconvert import HTMLExporter
from .loader import show
from .paths import stems, Glob, parent, P, stem
from .markup import read_json, writelines, makedir
from .logger import Info, Warn
from .loader import show, pd
from itertools import chain, cycle

# %% ../nbs/jupyter_notebook.ipynb 3
def is_in_notebook():
    try:
        # Test adapted from tqdm.autonotebook: https://github.com/tqdm/tqdm/blob/master/tqdm/autonotebook.py
        get_ipython = sys.modules["IPython"].get_ipython
        if "IPKernelApp" not in get_ipython().config:
            raise ImportError("console")

        return importlib.util.find_spec("IPython") is not None
    except (AttributeError, ImportError, KeyError):
        return False

# %% ../nbs/jupyter_notebook.ipynb 4
def save_notebook(file_path):
    start_md5 = hashlib.md5(open(file_path, "rb").read()).hexdigest()
    display(Javascript("IPython.notebook.save_checkpoint();"))
    current_md5 = start_md5

    while start_md5 == current_md5:
        time.sleep(1)
        current_md5 = hashlib.md5(open(file_path, "rb").read()).hexdigest()
    Info(f"Saved the notebook at {file_path}!")


def backup_this_notebook(
    this_file_path,
    save_html_to=None,
    override_previous_backup=False,
    changelog=None,
    exclude_input=False,
    force_save_notebook=True,
):
    if save_html_to is None:
        save_html_to = (
            parent(P(this_file_path)).resolve() / f"backups/{stem(this_file_path)}"
        )
        files = [f for f in stems(save_html_to) if f.split("__")[-1].isdigit()]
        available_number = max([int(i.split("__")[-1]) for i in files], default=-1) + 1
        save_to = f"{save_html_to}/{stem(this_file_path)}__{available_number:04}.html"
    if override_previous_backup:
        if available_number != 0:
            available_number -= 1
        if (
            input(
                f"Are you sure you want to override `{save_html_to}/{stem(this_file_path)}__{available_number:04}.html` ? [y/n]"
            ).lower()
            != "y"
        ):
            raise ValueError("Aborting")
        save_to = f"{save_html_to}/{stem(this_file_path)}__{available_number:04}.html"
    Info(f"Backing up this version of notebook to {save_to}")
    if force_save_notebook:
        save_notebook(this_file_path)
    this_notebook = nbformat.reads(
        json.dumps(read_json(this_file_path)),
        as_version=4,
    )

    html_exporter = HTMLExporter(template_name="classic")
    if exclude_input:
        html_exporter.exclude_input = True
    (body, resources) = html_exporter.from_notebook_node(this_notebook)
    makedir(save_html_to)
    writelines([body], save_to)
    if changelog is None:
        Warn(
            "Use `changelog` argument to the devs know what is important in the backup"
        )
        changelog = ""
    changelog_file = P(save_html_to) / "changelog.md"
    changelog_file.touch()
    changelog = f"\n## {stem(save_to)}\n{changelog}"
    changelog_file.write_lines(changelog.split("\n"), mode="a+")
    Info(f"Success! Visit {changelog_file} for detailed changes")
    return save_to


def backup_all_notebooks(
    folder,
):
    all_notebooks = P(folder).Glob("*.ipynb")
    for notebook in all_notebooks:
        backup_this_notebook(notebook, force_save_notebook=False)

# %% ../nbs/jupyter_notebook.ipynb 6
def display_dfs_side_by_side(*args, titles=cycle([""]), max_rows=50):
    html_str = ""
    for df, title in zip(args, chain(titles, cycle(["</br>"]))):
        html_str += '<th style="text-align:center"><td style="vertical-align:top">'
        html_str += f'<h2 style="text-align: center;">{title}</h2>'
        html_str += df.to_html(max_rows=max_rows).replace(
            "table", 'table style="display:inline"'
        )
        html_str += "</td></th>"
    display_html(html_str, raw=True)


def show_big_dataframe(df, max_rows=30):
    with pd.option_context(
        "display.max_columns",
        1000,
        "display.max_colwidth",
        1000,
        "display.max_rows",
        max_rows,
    ):
        show(df, max_rows=max_rows)

# %% ../nbs/jupyter_notebook.ipynb 7
def h1(text):
    show(Markdown(f"## {text}"))


def h2(text):
    show(Markdown(f"## {text}"))


def h3(text):
    show(Markdown(f"### {text}"))


def h4(text):
    show(Markdown(f"#### {text}"))


def h5(text):
    show(Markdown(f"##### {text}"))


def h6(text):
    show(Markdown(f"###### {text}"))

# %% ../nbs/jupyter_notebook.ipynb 8
# Function to shut down the current notebook session
def shutdown_current_notebook(delay: int = None):
    if delay:
        time.sleep(delay)
    os.kill(os.getpid(), 9)



from ..loader import *
from ..torch_loader import *
from fastcore.basics import ifnone
from functools import partial
from ..markup2 import AD


class IOHook:
    def __init__(self, module, depth=None, kwarg_hooks=None):
        self.module = module
        self.hook_handles = []
        self.depth = ifnone(depth, 1)
        self.kwarg_hooks = ifnone(kwarg_hooks, {})
        self.register_hooks()

    def hook_fn(self, module, input, kwargs, output, name):
        Info(f"{name} ({module.__class__.__name__})")
        log_depth = 2
        if hasattr(module, "weight"):
            Info(f"Weights: {module.weight}", depth=log_depth)
        if hasattr(module, "bias"):
            Info(f"Biases: {module.bias}", depth=log_depth)
        if input:
            Info(f"Input: {input}", depth=log_depth)
        if kwargs:
            Info(f"KWargs: {kwargs}", depth=log_depth)
        Info(f"Output: {output}", depth=log_depth)
        line()

    def register_hooks_recursive(self, submodule, prefix, current_depth=1):
        if self.depth is None or current_depth <= self.depth:
            hook_handle = submodule.register_forward_hook(
                partial(self.hook_fn, name=prefix),
                with_kwargs=True,
            )
            Info(f"Registered {submodule.__class__.__name__} @ {prefix}")
            self.hook_handles.append(hook_handle)

        if self.depth is None or current_depth < self.depth:
            for name, subsubmodule in submodule.named_children():
                self.register_hooks_recursive(
                    subsubmodule,
                    prefix=prefix + "." + name,
                    current_depth=current_depth + 1,
                )

    def register_hooks(self):
        self.register_hooks_recursive(self.module, prefix="")

    def remove_hooks(self):
        for handle in self.hook_handles:
            handle.remove()


# skip = {'SiLU', 'LayerNorm'}
# keep = {'CrossAttnDownBlock2D', 'CrossAttnUpBlock2D', 'UNetMidBlock2DCrossAttn', 'UpBlock2D', 'DownBlock2D'}
def print_shapes_hook(module, input, kwargs, output, skip=None, keep=None):
    skip = set() if skip is None else set(skip)  # Modules to exclude while printing
    keep = set() if keep is None else set(keep)  # Modules to include while printing
    try:
        name = module.__class__.__name__
        if name in skip:
            return
        if name not in keep:
            return
        print(f"Module Name: {name}")
        print(f"Input Kwargs: ")
        print(AD(kwargs).summary(depth=1))
        print(f"Input Args: {input}")
        print("Outputs: ")
        if isinstance(output, (list, tuple, set)):
            output = {str(i + 1): v for i, v in enumerate(output)}
            print(AD(output).summary(depth=1))
        else:
            print(output)
        line()
    except Exception as e:
        print(f"ERROR: {e} @ {name}")


def attach_hooks(model):
    """Function to attach the hooks and return handles"""
    handles = []
    for layer in model.children():
        handle = layer.register_forward_hook(print_shapes_hook, with_kwargs=True)
        handles.append(handle)
        if len(list(layer.children())) > 0:
            handles.extend(attach_hooks(layer))
    return handles


def detach_hooks(handles):
    """Function to detach hooks"""
    for handle in handles:
        handle.remove()



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/logging.ipynb.

# %% auto 0
__all__ = ['console', 'print', 'reset_logger_width', 'logger', 'Debug', 'Info', 'Warn', 'Excep', 'warn_mode', 'info_mode',
           'debug_mode', 'excep_mode', 'in_warn_mode', 'in_info_mode', 'in_debug_mode', 'in_excep_mode', 'frames',
           'get_console', 'reset_logger', 'enter_exit', 'get_logger_level', 'logger_mode', 'in_logger_mode',
           'notify_waiting']

# %% ../nbs/logging.ipynb 4
from rich.console import Console
from rich.theme import Theme
from loguru import logger
from datetime import datetime
from fastcore.basics import patch_to, ifnone
from rich.logging import RichHandler
from pathlib import Path
from contextlib import contextmanager
from rich._spinners import SPINNERS
import random

# from torch_snippets.ipython import is_in_notebook

from functools import wraps
import time

# %% ../nbs/logging.ipynb 5
def get_console(width=None):
    return Console(
        width=width,
        theme=Theme(
            {
                "repr.number": "bold cyan",
                "repr.string": "bold green",
                "logging.level.info": "dim yellow",
                "logging.level.warning": "dim red",
                "logging.level.exception": "bold red",
            }
        ),
    )


console = get_console()
print = console.print

# %% ../nbs/logging.ipynb 8
@patch_to(RichHandler)
def render(
    self,
    *,
    record,
    traceback,
    message_renderable: "ConsoleRenderable",
) -> "ConsoleRenderable":
    """patched the renderer to print function name as well"""
    path = Path(record.pathname).name
    level = self.get_level_text(record)
    time_format = None if self.formatter is None else self.formatter.datefmt
    log_time = datetime.fromtimestamp(record.created)

    log_renderable = self._log_render(
        self.console,
        [message_renderable] if not traceback else [message_renderable, traceback],
        log_time=log_time,
        time_format=time_format,
        level=level,
        path=path,
        line_no=f"{record.funcName}:{record.lineno}",
        link_path=f"{record.pathname}:{record.lineno}"
        if self.enable_link_path
        else None,
    )
    return log_renderable


def reset_logger(level="INFO", width=172, silent=True, disable_stdout=False):
    if level is not None:
        [logger.remove() for _ in range(100)]
        if not disable_stdout:
            logger.configure(
                handlers=[
                    {
                        "sink": RichHandler(
                            rich_tracebacks=True,
                            console=console,
                            tracebacks_show_locals=False,
                        ),
                        "format": "<level>{message}</level>",
                        "backtrace": True,
                        "level": level.upper(),
                    }
                ],
            )
    if width is not None:
        for handler_id in logger._core.handlers:
            try:
                handler = logger._core.handlers[handler_id]
                handler._sink._handler.console = get_console(width=width)
            except:
                ...
    if not silent:
        logger.info(f"reset logger's console width to {width} and level to {level}!")


reset_logger_width = lambda width: reset_logger(width=width)

reset_logger()

logger = logger

Debug = lambda *x, depth=0: logger.opt(depth=depth + 1).log(
    "DEBUG", x[0] if len(x) == 1 else "; ".join([str(i) for i in x])
)
Info = lambda *x, depth=0: logger.opt(depth=depth + 1).log(
    "INFO", x[0] if len(x) == 1 else "; ".join([str(i) for i in x])
)
Warn = lambda *x, depth=0: logger.opt(depth=depth + 1).log(
    "WARNING", x[0] if len(x) == 1 else "; ".join([str(i) for i in x])
)
Excep = lambda *x, depth=0: logger.opt(depth=depth + 1).log(
    "ERROR", x[0] if len(x) == 1 else "; ".join([str(i) for i in x])
)

# %% ../nbs/logging.ipynb 11
def enter_exit(func):
    """
    Logs the time taken to execute a function along with entry & exit time stamps
    """
    logger_ = logger.opt(depth=1)

    @wraps(func)
    def function_timer(*args, **kwargs):
        tic = time.time()
        logger_.log("DEBUG", f"Entered function `{func.__name__}`")
        o = func(*args, **kwargs)
        toc = time.time()
        logger_.log(
            "DEBUG", f"Exiting function `{func.__name__}` after {toc-tic:.3f} seconds"
        )
        return o

    return function_timer

# %% ../nbs/logging.ipynb 14
def get_logger_level():
    lv = [
        l
        for l, v in logger._core.levels_lookup.items()
        if v[2] == logger._core.min_level
    ][0]
    return lv.lower()


@contextmanager
def logger_mode(level):
    lv = get_logger_level()
    reset_logger(level.upper())
    yield
    reset_logger(lv.upper())


def in_logger_mode(level):
    return get_logger_level() == level


warn_mode = lambda: logger_mode("warning")
info_mode = lambda: logger_mode("info")
debug_mode = lambda: logger_mode("debug")
excep_mode = lambda: logger_mode("error")

in_warn_mode = lambda: in_logger_mode("warning")
in_info_mode = lambda: in_logger_mode("info")
in_debug_mode = lambda: in_logger_mode("debug")
in_excep_mode = lambda: in_logger_mode("error")

# %% ../nbs/logging.ipynb 17
frames = [
    "Guess the anime (while you wait): 🍥🍃🔥🥋🦊🍜🌆🌄    ",  # Naruto
    "Guess the anime (while you wait): 🐉💥🥋🚀🔥🐲🌌🍃    ",  # Dragon Ball Z
    "Guess the anime (while you wait): 👒☠️⛵🏴‍☠️🍖🍗🌊🏝️🐒  ",  # One Piece
    "Guess the anime (while you wait): 🪶🏰🧑‍🔬🦿🪓🌅🐦👹    ",  # Attack on Titan
    "Guess the anime (while you wait): 📓✒️🧑‍⚖️☠️🕊️🔍🌃      ",  # Death Note
    "Guess the anime (while you wait): 🔧🔮⚗️🤖👁️‍🗨️💥🔥      ",  # Fullmetal Alchemist
    "Guess the anime (while you wait): 🦸‍♂️🦸‍♀️💥🏫🌟🌆🔥      ",  # My Hero Academia
    "Guess the anime (while you wait): 🤖🌃🌌🛰️🪐👁️‍🗨️💥      ",  # Neon Genesis Evangelion
    "Guess the anime (while you wait): 🤠🚀🌌🎷🌟🔫💰      ",  # Cowboy Bebop
    "Guess the anime (while you wait): 👧🐉🏯🌟🍜🛁🌌      ",  # Spirited Away
    "Guess the movie (while you wait): 🌌🚀🌠🪐🤖👽🔱✨🚁🌑",  # Star Wars
    "Guess the movie (while you wait): 🧙🏰🌋🗡️👑🐉🪶🍃🦅💍",  # The Lord of the Rings
    "Guess the movie (while you wait): 🧙‍♂️⚡🏰🪄🦉🔮🚂🐍🧹📚",  # Harry Potter
    "Guess the movie (while you wait): 🌿🌏🚁💎💫🌞🦋🐉🍃🔶",  # Avatar
    "Guess the movie (while you wait): 🏴‍☠️⚓🦜🌊💰🚢🔫🌴🗺️🔪",  # Pirates of the Caribbean
    "Guess the movie (while you wait): 🕶️🌐💊🏙️🧑‍💻🔫🪑🕊️🦹‍♂️🔵",  # The Matrix
    "Guess the movie (while you wait): 🦸‍♂️🦸‍♀️🌟🦹‍♂️🦹‍♀️💥🌆🤖🔨🛡️",  # The Avengers
    "Guess the movie (while you wait): 🦖🦕🌴🪶🪶🪶🚁🧑‍🔬🪶🧑‍🔬",  # Jurassic Park
    "Guess the movie (while you wait): 🦁👑🌅🌴🐗🐾🎶👑🦏🐘",  # The Lion King
    "Guess the movie (while you wait): 🧸🤠🚀🧒🐍🪁🎈🎉🤖👾",  # Toy Story
    "Guess the movie (while you wait): ❄️👑⛄❄️🧝‍♀️🦌🏰🎶🏔️🌬️",  # Frozen
    "Guess the movie (while you wait): 🚢🌊❄️💔🎻🔱⚓🚢🌟🌅",  # Titanic
    "Guess the movie (while you wait): 🚲👽🌕🌱🌌🌠🌟🪐👨‍🔬🛸",  # E.T. the Extra-Terrestrial
    "Guess the movie (while you wait): 📜🗝️🏢🔓🧱🧔💰🚶‍♂️⛏️🌅",  # The Shawshank Redemption
    "Guess the movie (while you wait): 🏃🌳🎓🏈🍫🏅🍤🚌🍤🦐",  # Forrest Gump
    "Guess the movie (while you wait): 🍊🎩👨‍👩‍👦🔫🍷🇮🇹🕶️📝🚬💔",  # The Godfather
]


@contextmanager
def notify_waiting(message):
    random.shuffle(frames)
    SPINNERS["guess"] = {"interval": 3000, "frames": frames}

    status = console.status(f"[red]\n{message:10}", spinner="clock")
    with status as _:
        s = time.perf_counter()
        yield
        time_taken = time.perf_counter() - s
        time.sleep(0.1)
    Info(f"{message} - Completed in {time_taken:.2f} s", depth=1)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/paths.ipynb.

# %% auto 0
__all__ = ['P', 'dill', 'input_to_str', 'output_to_path', 'isdir', 'makedir', 'fname', 'fname2', 'stem', 'stems', 'parent',
           'extn', 'Glob', 'find', 'zip_files', 'unzip_file', 'list_zip', 'md5', 'remove_duplicates', 'common_items',
           'folder_summary', 'readlines', 'writelines', 'rename_batch', 'dumpdill', 'loaddill']

# %% ../nbs/paths.ipynb 2
from fastcore.basics import patch_to
from fastcore.foundation import L
from functools import wraps
from torch_snippets.loader import (
    choose as ts_choose,
    Tqdm,
    os,
    logger,
    Info,
    Debug,
    Warn,
    Excep,
    common,
)
from pathlib import Path
import hashlib, shutil
import glob
import dill, time

# %% ../nbs/paths.ipynb 3
def input_to_str(func):
    @wraps(func)
    def inner(input, *args, **kwargs):
        if isinstance(input, P):
            input = str(input)
        if isinstance(input, list):
            input = [str(i) for i in input]
        out = func(input, *args, **kwargs)
        return out

    return inner


def output_to_path(func):
    @wraps(func)
    def inner(input, *args, **kwargs):
        out = func(input, *args, **kwargs)
        if isinstance(out, str):
            out = P(out)
        if isinstance(out, list):
            if len(out) > 0 and isinstance(out[0], str):
                out = [P(o) for o in out]
        return out

    return inner

# %% ../nbs/paths.ipynb 4
P = Path
P.ls = lambda self: L(self.iterdir())
P.__repr__ = lambda self: f"» {self}"


@patch_to(P)
def rmtree(self, prompt="Really remove `{self}` and its contents? [y/n] ", force=False):
    if force:
        shutil.rmtree(self)
    elif prompt and input(prompt.format(self=self)).lower() == "y":
        shutil.rmtree(self)
    else:
        raise OSError(f"{self} exists and is not empty")


@patch_to(P)
def size(self):
    if self.is_dir():
        raise Exception(f"`{self}` is a directory")
    fsize = os.path.getsize(self) >> 20
    fsize = f"{fsize} MB" if fsize > 0 else f"{os.path.getsize(self) >> 10} KB"
    return fsize


@patch_to(P)
def extn(self, pattern="*"):
    return self.suffix.replace(".", "")


@patch_to(P)
def Glob(self, pattern="*"):
    return L(self.glob(pattern))


@patch_to(P)
def sample(self, pattern="*"):
    return ts_choose(self.Glob(pattern))


@patch_to(P)
def mv(self, to):
    os.rename(self, to)
    return P(to)


import shutil


@patch_to(P)
def cp(self, to):
    return P(shutil.copy(self, to))


@patch_to(P)
def rm(
    self,
    confirm_prompt="Are you sure you want to delete `{self}`? [y/N]",
    silent=True,
    missing_ok=True,
):
    confirm = input(confirm_prompt.format(self=self)) if confirm_prompt else "y"
    if confirm.lower() == "y":
        if missing_ok:
            try:
                os.remove(self)
            except:
                ...
        else:
            os.remove(self)
        if not silent:
            logger.info(f"Deleted {self}")
    else:
        if not silent:
            logger.info(f"Aborting delete: {self}")

# %% ../nbs/paths.ipynb 20
def isdir(fpath):
    return os.path.isdir(fpath)


@input_to_str
def makedir(x, prompt=False, silent=True):
    create = input(f"Creating folder {x}. Are you sure? [y/n]") if prompt else "y"
    if create:
        os.makedirs(x, exist_ok=True)
    if not silent:
        Info(f"Created folder {x}")


@input_to_str
def fname(fpath):
    return fpath.split("/")[-1]


@input_to_str
def fname2(fpath):
    return stem(fpath.split("/")[-1])


def stem(fpath):
    return P(fpath).stem


@input_to_str
def stems(folder, silent=False):
    if isinstance(folder, (str, P)):
        return L([stem(str(x)) for x in Glob(folder, silent=silent)])
    if isinstance(folder, list):
        return L([stem(x) for x in folder])


P.stems = lambda self: stems(self.ls())


@input_to_str
@output_to_path
def parent(fpath):
    out = "/".join(fpath.split("/")[:-1])
    if out == "":
        return "./"
    else:
        return out


def extn(x):
    return P(x).extn()


@input_to_str
@output_to_path
def Glob(x, extns=None, silent=False):
    files = glob.glob(x + "/*") if "*" not in x else glob.glob(x)
    if extns:
        if isinstance(extns, str):
            extns = extns.split(",")
        files = [f for f in files if any([f.endswith(ext) for ext in extns])]

    # if not silent: logger.opt(depth=1).log('INFO', '{} files found at {}'.format(len(files), x))
    return files


def find(
    item=None, List=None, match_stem=False, condition=None, return_indexes_also=False
):
    """Find an `item` in a `List`
    >>> find('abc', ['ijk','asdfs','dfsabcdsf','lmnop'])
    'dgsabcdsf'
    >>> find('file1', ['/tmp/file0.jpg', '/tmp/file0.png', '/tmp/file1.jpg', '/tmp/file1.png', '/tmp/file2.jpg', '/tmp/file2.png'])
    ['/tmp/file1.jpg', '/tmp/file1.png']
    """
    if callable(condition):
        filtered = [(ix, i) for ix, i in enumerate(List) if condition(i)]

    else:
        filtered = [(ix, i) for ix, i in enumerate(List) if item in str(i)]

    if match_stem and len(filtered) > 1:
        filtered = [(ix, f) for ix, f in filtered if stem(f) == item]

    if len(filtered) == 1:
        if return_indexes_also:
            return filtered[0]
        else:
            return filtered[0][1]
    elif len(filtered) == 0:
        return None
    else:
        ixs, filtered_items = list(zip(*filtered))
        if return_indexes_also:
            return ixs, filtered_items
        else:
            return filtered_items

# %% ../nbs/paths.ipynb 22
import zipfile
import tarfile


def zip_files(list_of_files, dest):
    dest = str(dest)
    logger.info(f"Zipping {len(list_of_files)} files to {dest}...")
    if dest.lower().endswith(".zip"):
        with zipfile.ZipFile(dest, "w") as zipMe:
            for file in Tqdm(list_of_files):
                zipMe.write(file, compress_type=zipfile.ZIP_DEFLATED)
    elif dest.lower().endswith(".tar.gz"):
        with tarfile.open(dest, "w:gz") as tarMe:
            for file in Tqdm(list_of_files):
                tarMe.add(file)
    return P(dest)


def unzip_file(file, dest):
    file = str(file)
    if file.lower().endswith(".zip"):
        with zipfile.ZipFile(file, "r") as zip_ref:
            zip_ref.extractall(dest)
    elif file.lower().endswith(".tar.xz") or file.endswith(".tar.gz"):
        with tarfile.open(file, "r") as f:
            f.extractall(dest)
    return P(dest)


def list_zip(file):
    elements = []
    with zipfile.ZipFile(file, "r") as zipObj:
        listOfiles = zipObj.namelist()
        for elem in listOfiles:
            elements.append(elem)
    return elements

# %% ../nbs/paths.ipynb 24
def md5(fname):
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def remove_duplicates(files):
    """Check a list of files and remove duplicates based on their checksum"""
    import pandas as pd
    from torch_snippets.loader import diff

    hashes = [md5(f) for f in files]
    df = pd.DataFrame({"f": files, "h": hashes})
    x = df.drop_duplicates("h")
    y = diff(files, x.f)
    for i in y:
        os.rename(i, "./x")
    # !rm ./x
    return


def common_items(*fldrs, verbose=True):
    fldr_items = [stems(fldr) for fldr in fldrs]
    o = sorted(common(*fldr_items))
    if verbose:
        Info(
            f"Returning {len(o)} common items from folders of {[len(_f) for _f in fldr_items]} items each"
        )
    return o


def folder_summary(thing):
    things = Glob(thing)
    info = []
    for thing in things:
        if thing.is_dir():
            info.append(f"{thing} - {len(Glob(thing))} items")
        else:
            info.append(f"{thing} - {thing.size()}")
    return "\n".join(info)

# %% ../nbs/paths.ipynb 26
def readlines(fpath, silent=False, encoding=None):
    with open(fpath, "r", encoding=encoding) as f:
        lines = f.read().split("\n")
        lines = [l.strip() for l in lines if l.strip() != ""]
        if not silent:
            logger.opt(depth=1).log("INFO", f"loaded {len(lines)} lines")
        return lines


@patch_to(P)
def read_lines(self, silent=False, encoding=None):
    return readlines(self, silent=silent, encoding=encoding)


def writelines(lines, file, mode="w"):
    makedir(parent(file))
    failed = []
    with open(file, mode) as f:
        for line in lines:
            try:
                f.write(f"{line}\n")
            except:
                failed.append(line)
    if failed != []:
        logger.opt(depth=1).log(
            "INFO", f"Failed to write {len(failed)} lines out of {len(lines)}"
        )
        return failed


@patch_to(P)
def write_lines(self, lines, mode):
    return writelines(lines, self, mode)

# %% ../nbs/paths.ipynb 28
def rename_batch(folder, func, debug=False, one_file=False):
    from torch_snippets.loader import now

    "V.V.Imp: Use debug=True first to confirm file name changes are as expected"
    if isinstance(folder, (str, P)):
        folder = Glob(folder)
    sources = []
    destins = []
    log_file = f"moved_files_{now()}.log"
    for f in folder:
        source = f
        destin = func(f)
        if source == destin:
            continue
        if debug:
            logger.debug(f"moving `{source}` --> `{destin}`")
        else:
            # !mv {source.replace(' ','\ ')} {destin.replace(' ','\ ')}
            logger.info(f"moving `{source}` --> `{destin}`")
            os.rename(source, destin)
        # !echo {source.replace(' ','\ ')} --\> {destin.replace(' ','\ ')} >> {logfile}
        if one_file:
            break

# %% ../nbs/paths.ipynb 29
dill = dill


def dumpdill(obj, fpath, silent=False):
    """Dump a python object as a dill file (better replacement to pickle)"""
    start = time.time()
    fpath = P(fpath)
    fpath.parent.mkdir(exist_ok=True)
    with open(fpath, "wb") as f:
        dill.dump(obj, f)
    if not silent:
        fsize = fpath.size()
        logger.opt(depth=1).log(
            "INFO",
            f'Dumped object of size ≈{fsize} @ "{fpath}" in {time.time()-start:.2e} seconds',
        )
    return P(fpath)


def loaddill(fpath):
    """Load a python object from a dill file"""
    fpath = str(fpath)
    with open(fpath, "rb") as f:
        obj = dill.load(f)
    return obj



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/bounding_boxes.ipynb.

# %% auto 0
__all__ = ['randint', 'BB', 'df2bbs', 'bbs2df', 'bbfy', 'jitter', 'compute_eps', 'enlarge_bbs', 'shrink_bbs', 'iou',
           'compute_distance_matrix', 'compute_distances', 'split_bb_to_xyXY', 'combine_xyXY_to_bb', 'is_absolute',
           'is_relative', 'to_relative', 'to_absolute', 'merge_by_bb', 'isin']

# %% ../nbs/bounding_boxes.ipynb 2
import numpy as np
import pandas as pd
from PIL.Image import Image
from typing import Tuple

# %% ../nbs/bounding_boxes.ipynb 3
randint = np.random.randint


class BB:
    """A Bounding Box defined by the top-left and bottom-right coordinates"""

    def __init__(self, *bb):
        # assert len(bb) == 4, 'expecting a list/tuple of 4 values respectively for (x,y,X,Y)'
        if len(bb) == 4:
            x, y, X, Y = bb
        elif len(bb) == 1:
            ((x, y, X, Y),) = bb
        rel = True if max(x, y, X, Y) < 1 else False
        # if not rel:
        #    x, y, X, Y = map(lambda i: int(round(i)), (x, y, X, Y))
        self.bb = x, y, X, Y
        self.x, self.y, self.X, self.Y = x, y, X, Y
        self.xc, self.yc = (self.x + self.X) / 2, (self.y + self.Y) / 2
        self.c = (self.xc, self.yc)
        self.h = Y - y
        self.w = X - x
        self.area = self.h * self.w
        self.shape = (self.h, self.w)

    def __getitem__(self, i):
        return self.bb[i]

    def __repr__(self):
        return self.bb.__repr__()

    def __len__(self):
        return 4

    def __eq__(self, other):
        return (
            self.x == other.x
            and self.y == other.y
            and self.X == other.X
            and self.Y == other.Y
        )

    def __hash__(self):
        return hash(tuple(self))

    def __add__(self, origin):
        a, b = origin[:2]
        x, y, X, Y = self
        return BB(x + a, y + b, X + a, Y + b)

    def remap(self, og_dim: Tuple[int, int], new_dim: Tuple[int, int]):
        """
        og_dim = (Height, Width)
        new_dim = (Height, Width)
        """
        h, w = og_dim
        H, W = new_dim
        sf_x = H / h
        sf_y = W / w
        return BB(
            round(sf_x * self.x),
            round(sf_y * self.y),
            round(sf_x * self.X),
            round(sf_y * self.Y),
        )

    def relative(self, dim: Tuple[int, int]):
        h, w = dim
        return BB(self.x / w, self.y / h, self.X / w, self.Y / h)

    def absolute(self, dim: Tuple[int, int]):
        h, w = dim
        return BB(int(self.x * w), int(self.y * h), int(self.X * w), int(self.Y * h))

    def local_to(self, _bb):
        x, y, X, Y = self
        a, b, A, B = _bb
        return BB(x - a, y - b, X - a, Y - b)

    def jitter(self, noise, preserve_shape=True):
        if isinstance(noise, (int, float)):
            return BB([i + (noise - randint(2 * noise)) for i in self])
        elif isinstance(noise, (list, tuple)):
            if len(noise) == 2:
                dx, dy = noise
                dx, dy, dX, dY = dx / 2, dy / 2, dx / 2, dy / 2
            elif len(noise) == 4:
                dx, dy, dX, dY = noise
            if 0 < dx < 1:
                dx = int(self.w * dx)
            if 0 < dX < 1:
                dX = int(self.w * dX)
            if 0 < dy < 1:
                dy = int(self.h * dy)
            if 0 < dY < 1:
                dY = int(self.w * dY)
            dx = dx - 2 * randint(dx + 1)
            dy = dy - 2 * randint(dy + 1)
            if preserve_shape:
                dX = dx
                dY = dy
            else:
                dX = dX - 2 * randint(dX + 1)
                dY = dy - 2 * randint(dY + 1)
            dbb = BB(dx, dy, dX, dY)
            return BB([max(0, i + j) for i, j in zip(self, dbb)])

    def shrink_inplace(self):
        "return a new thing, shrunk"

    def add_padding(self, *pad):
        if len(pad) == 4:
            _x, _y, _X, _Y = pad
        else:
            (pad,) = pad
            _x, _y, _X, _Y = pad, pad, pad, pad
        x, y, X, Y = self.bb
        return max(0, x - _x), max(0, y - _y), X + _x, Y + _y

    def l2(self, other, xyfactor=(1, 1)):
        _x_, _y_ = xyfactor
        other = BB(other)
        xc, yc = self.xc, self.yc
        ac, bc = other.xc, other.yc
        return np.sqrt(_x_ * (xc - ac) ** 2 + _y_ * (yc - bc) ** 2)

    def distances(self, other_bbs, threshold=None, direction=None):
        other_bbs = bbfy(other_bbs)
        if direction:
            assert direction in "x,y,left,right,top,down".split(",")
            if direction == "x":
                output = [
                    (ix, bb, self.l2(bb, xyfactor=(1, 1000))) for (ix, bb) in other_bbs
                ]
                return pd.DataFrame(output, columns="ix,bb,dist".split(","))
            raise NotImplementedError("")
        return sorted(other_bbs, key=lambda obj: self.l2(obj[1]))

# %% ../nbs/bounding_boxes.ipynb 8
def df2bbs(df):
    if "bb" in df.columns:
        try:
            return bbfy(df["bb"].values.tolist())
        except:
            return bbfy(df["bb"].map(lambda x: eval(x)).values.tolist())
    return [BB(bb) for bb in df[list("xyXY")].values.tolist()]


def bbs2df(bbs):
    bbs = [list(bb) for bb in bbs]
    return pd.DataFrame(bbs, columns=["x", "y", "X", "Y"])


def bbfy(bbs):
    return [BB(bb) for bb in bbs]


def jitter(bbs, noise):
    return [BB(bb).jitter(noise) for bb in bbs]


def compute_eps(eps):
    if isinstance(eps, tuple):
        if len(eps) == 4:
            epsx, epsy, epsX, epsY = eps
        else:
            epsx, epsy = eps
            epsx, epsy, epsX, epsY = epsx / 2, epsy / 2, epsx / 2, epsy / 2
    else:
        epsx, epsy, epsX, epsY = eps / 2, eps / 2, eps / 2, eps / 2
    return epsx, epsy, epsX, epsY


def enlarge_bbs(bbs, eps=0.2):
    "enlarge all `bbs` by `eps` fraction (i.e., eps*100 percent)"
    bbs = bbfy(bbs)
    epsx, epsy, epsX, epsY = compute_eps(eps)
    bbs = bbfy(bbs)
    shs = [(bb.h, bb.w) for bb in bbs]
    return [
        BB(x - (w * epsx), y - (h * epsy), X + (w * epsX), Y + (h * epsY))
        for (x, y, X, Y), (h, w) in zip(bbs, shs)
    ]


def shrink_bbs(bbs, eps=0.2):
    "shrink all `bbs` by `eps` fraction (i.e., eps*100 percent)"
    bbs = bbfy(bbs)
    epsx, epsy, epsX, epsY = compute_eps(eps)
    bbs = bbfy(bbs)
    shs = [(bb.h, bb.w) for bb in bbs]
    return [
        BB(x + (w * epsx), y + (h * epsy), X - (w * epsX), Y - (h * epsY))
        for (x, y, X, Y), (h, w) in zip(bbs, shs)
    ]

# %% ../nbs/bounding_boxes.ipynb 9
def iou(bboxes1, bboxes2):
    bboxes1 = np.array(bboxes1)
    bboxes2 = np.array(bboxes2)
    x11, y11, x12, y12 = np.split(bboxes1, 4, axis=1)
    x21, y21, x22, y22 = np.split(bboxes2, 4, axis=1)
    xA = np.maximum(x11, np.transpose(x21))
    yA = np.maximum(y11, np.transpose(y21))
    xB = np.minimum(x12, np.transpose(x22))
    yB = np.minimum(y12, np.transpose(y22))
    interArea = np.maximum((xB - xA), 0) * np.maximum((yB - yA), 0)
    boxAArea = (x12 - x11) * (y12 - y11)
    boxBArea = (x22 - x21) * (y22 - y21)
    iou = interArea / (boxAArea + np.transpose(boxBArea) - interArea)
    return iou


def compute_distance_matrix(bboxes1, bboxes2):
    # Convert the bounding box lists to NumPy arrays
    bboxes1 = np.array(bboxes1)
    bboxes2 = np.array(bboxes2)

    # Extract the x, y coordinates of the bounding boxes
    xy1 = bboxes1[:, :2]
    xy2 = bboxes2[:, :2]

    # Compute the squared Euclidean distances between all pairs of bounding box coordinates
    distance_matrix = np.sum((xy1[:, np.newaxis] - xy2) ** 2, axis=-1)

    # Take the square root to get the Euclidean distances
    distance_matrix = np.sqrt(distance_matrix)

    return distance_matrix


def compute_distances(df1, df2, shrink_factors=(1, 1)):
    """Return euclidean distance mxn matrix for all boxes from df1 with all boxes from df2"""
    sx, sy = shrink_factors
    bbs1 = np.array(df2bbs(df1)) / np.array([sx, sy, sx, sy])
    bbs2 = np.array(df2bbs(df2)) / np.array([sx, sy, sx, sy])

    distances = compute_distance_matrix(bbs1, bbs2)
    return distances

# %% ../nbs/bounding_boxes.ipynb 10
def split_bb_to_xyXY(df):
    "convert bb column to separate x,y,X,Y columns"
    df = df.copy()
    assert isinstance(df, pd.DataFrame)
    if all([item in df.columns for item in "xyXY"]):
        return df
    assert "bb" in df.columns, "Expecting the df's bounding boxes to be in `bb` column"
    try:
        df["bb"] = df["bb"].map(eval)
    except:
        pass
    df["x"] = df["bb"].map(lambda x: x[0])
    df["y"] = df["bb"].map(lambda x: x[1])
    df["X"] = df["bb"].map(lambda x: x[2])
    df["Y"] = df["bb"].map(lambda x: x[3])
    df.drop(["bb"], axis=1, inplace=True)
    return df


def combine_xyXY_to_bb(df):
    "combine `x,y,X,Y` to `bb` column"
    df = df.copy()
    assert all(
        [item in df.columns for item in "xyXY"]
    ), "All the columns `x`, `y`, `X`, `Y` should be in df"
    df["bb"] = df[[*"xyXY"]].values.tolist()
    df.drop([*"xyXY"], inplace=True, axis=1)
    return df


def is_absolute(df):
    bbs = df2bbs(df)
    bbs = np.array(bbs)
    return bbs.max() > 1.1


def is_relative(df):
    return not is_absolute(df)


def to_relative(df, height, width, force=False):
    if not force and is_relative(df):
        return df
    df = df.copy()
    if "x" not in df.columns and "bb" in df.columns:
        _recombine = True
        df = split_bb_to_xyXY(df)
    else:
        _recombine = False
    df["x"] = df["x"] / width
    df["y"] = df["y"] / height
    df["X"] = df["X"] / width
    df["Y"] = df["Y"] / height
    if _recombine:
        df = combine_xyXY_to_bb(df)
    return df


def to_absolute(df, height, width, force=False):
    if not force and is_absolute(df):
        return df
    df = df.copy()
    if "x" not in df.columns and "bb" in df.columns:
        _recombine = True
        df = split_bb_to_xyXY(df)
    else:
        _recombine = False
    df["x"] = (np.clip(df["x"], 0, 1) * width).astype(np.uint16)
    df["y"] = (np.clip(df["y"], 0, 1) * height).astype(np.uint16)
    df["X"] = (np.clip(df["X"], 0, 1) * width).astype(np.uint16)
    df["Y"] = (np.clip(df["Y"], 0, 1) * height).astype(np.uint16)
    if _recombine:
        df = combine_xyXY_to_bb(df)
    return df

# %% ../nbs/bounding_boxes.ipynb 18
def merge_by_bb(df1, df2, suffixes=("_x", "_y"), iou_threshold=0.1):
    """Merge df1 columns to df2 by using iou
    Make sure both df1 & df2 are relative or both absolute
    """
    # df1, df2 = [df.copy().reset_index(drop=True) for df in [df1, df2]]
    assert all([c in df1.columns for c in "xyXY"])
    assert all([c in df2.columns for c in "xyXY"])
    ious = iou(df2bbs(df1), df2bbs(df2))
    _isin = isin(df2bbs(df1), df2bbs(df2), return_matrix=True)
    _isin_r = isin(df2bbs(df2), df2bbs(df1), return_matrix=True)
    (ixs, jxs) = np.nonzero(ious)
    ious = ious[ixs, jxs]
    _df1 = df1.iloc[ixs]
    _df1.columns = [f"{c}{suffixes[0]}" for c in df1.columns]
    _df2 = df2.iloc[jxs]
    _df2.columns = [f"{c}{suffixes[1]}" for c in df2.columns]
    output = pd.concat(
        [
            _df1.reset_index(names=f"index_{suffixes[0]}"),
            _df2.reset_index(names=f"index_{suffixes[1]}"),
        ],
        axis=1,
    )
    output["iou"] = ious
    output["isin"] = _isin[ixs, jxs]
    output["isin_r"] = _isin_r.T[ixs, jxs]
    output = output.query("iou > @iou_threshold")
    return output


def isin(bboxes1, bboxes2, return_matrix=True):
    """return indexes of those boxes from `bboxes1` that are completely inside `bboxes2`"""
    bboxes1 = np.array(bboxes1)
    bboxes2 = np.array(bboxes2)
    x11, y11, x12, y12 = np.split(bboxes1, 4, axis=1)
    x21, y21, x22, y22 = np.split(bboxes2, 4, axis=1)
    xA = np.maximum(x11, np.transpose(x21))
    yA = np.maximum(y11, np.transpose(y21))
    xB = np.minimum(x12, np.transpose(x22))
    yB = np.minimum(y12, np.transpose(y22))
    interArea = np.maximum((xB - xA), 0) * np.maximum((yB - yA), 0)
    boxAArea = (x12 - x11) * (y12 - y11)
    output = interArea / boxAArea
    if return_matrix:
        return output
    ixs = np.where(output == 1)[0]
    return ixs



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/torch_snippets/',
                'doc_host': 'https://sizhky.github.io',
                'git_url': 'https://github.com/sizhky/torch_snippets/tree/master/',
                'lib_path': 'torch_snippets'},
  'syms': { 'torch_snippets.adapters': { 'torch_snippets.adapters._cvat_ann_2_csv': ( 'adapters.html#_cvat_ann_2_csv',
                                                                                      'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters._get_attribute_columns': ( 'adapters.html#_get_attribute_columns',
                                                                                             'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters._get_attribute_data': ( 'adapters.html#_get_attribute_data',
                                                                                          'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters._process': ('adapters.html#_process', 'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters.b64_2_file': ('adapters.html#b64_2_file', 'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters.b64_2_np': ('adapters.html#b64_2_np', 'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters.bytes_2_file': ( 'adapters.html#bytes_2_file',
                                                                                   'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters.csvs_2_cvat': ('adapters.html#csvs_2_cvat', 'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters.cvat_2_csvs': ('adapters.html#cvat_2_csvs', 'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters.file_2_bytes': ( 'adapters.html#file_2_bytes',
                                                                                   'torch_snippets/adapters.py'),
                                         'torch_snippets.adapters.np_2_b64': ('adapters.html#np_2_b64', 'torch_snippets/adapters.py')},
            'torch_snippets.bb_utils': { 'torch_snippets.bb_utils.BB': ('bounding_boxes.html#bb', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.__add__': ( 'bounding_boxes.html#bb.__add__',
                                                                                 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.__eq__': ( 'bounding_boxes.html#bb.__eq__',
                                                                                'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.__getitem__': ( 'bounding_boxes.html#bb.__getitem__',
                                                                                     'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.__hash__': ( 'bounding_boxes.html#bb.__hash__',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.__init__': ( 'bounding_boxes.html#bb.__init__',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.__len__': ( 'bounding_boxes.html#bb.__len__',
                                                                                 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.__repr__': ( 'bounding_boxes.html#bb.__repr__',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.absolute': ( 'bounding_boxes.html#bb.absolute',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.add_padding': ( 'bounding_boxes.html#bb.add_padding',
                                                                                     'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.distances': ( 'bounding_boxes.html#bb.distances',
                                                                                   'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.jitter': ( 'bounding_boxes.html#bb.jitter',
                                                                                'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.l2': ('bounding_boxes.html#bb.l2', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.local_to': ( 'bounding_boxes.html#bb.local_to',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.relative': ( 'bounding_boxes.html#bb.relative',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.remap': ('bounding_boxes.html#bb.remap', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.BB.shrink_inplace': ( 'bounding_boxes.html#bb.shrink_inplace',
                                                                                        'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.bbfy': ('bounding_boxes.html#bbfy', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.bbs2df': ('bounding_boxes.html#bbs2df', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.combine_xyXY_to_bb': ( 'bounding_boxes.html#combine_xyxy_to_bb',
                                                                                         'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.compute_distance_matrix': ( 'bounding_boxes.html#compute_distance_matrix',
                                                                                              'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.compute_distances': ( 'bounding_boxes.html#compute_distances',
                                                                                        'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.compute_eps': ( 'bounding_boxes.html#compute_eps',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.df2bbs': ('bounding_boxes.html#df2bbs', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.enlarge_bbs': ( 'bounding_boxes.html#enlarge_bbs',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.iou': ('bounding_boxes.html#iou', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.is_absolute': ( 'bounding_boxes.html#is_absolute',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.is_relative': ( 'bounding_boxes.html#is_relative',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.isin': ('bounding_boxes.html#isin', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.jitter': ('bounding_boxes.html#jitter', 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.merge_by_bb': ( 'bounding_boxes.html#merge_by_bb',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.shrink_bbs': ( 'bounding_boxes.html#shrink_bbs',
                                                                                 'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.split_bb_to_xyXY': ( 'bounding_boxes.html#split_bb_to_xyxy',
                                                                                       'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.to_absolute': ( 'bounding_boxes.html#to_absolute',
                                                                                  'torch_snippets/bb_utils.py'),
                                         'torch_snippets.bb_utils.to_relative': ( 'bounding_boxes.html#to_relative',
                                                                                  'torch_snippets/bb_utils.py')},
            'torch_snippets.bokeh_loader': { 'torch_snippets.bokeh_loader.get_bplot': ( 'bokeh_plotting.html#get_bplot',
                                                                                        'torch_snippets/bokeh_loader.py'),
                                             'torch_snippets.bokeh_loader.parse_sz': ( 'bokeh_plotting.html#parse_sz',
                                                                                       'torch_snippets/bokeh_loader.py')},
            'torch_snippets.charts': { 'torch_snippets.charts.confusion_matrix': ( 'charts.html#confusion_matrix',
                                                                                   'torch_snippets/charts.py')},
            'torch_snippets.dates': {},
            'torch_snippets.decorators': { 'torch_snippets.decorators.check_kwargs_not_none': ( 'decorators.html#check_kwargs_not_none',
                                                                                                'torch_snippets/decorators.py'),
                                           'torch_snippets.decorators.io': ('decorators.html#io', 'torch_snippets/decorators.py'),
                                           'torch_snippets.decorators.timeit': ('decorators.html#timeit', 'torch_snippets/decorators.py')},
            'torch_snippets.fastcores': {},
            'torch_snippets.icecream': {},
            'torch_snippets.imgaug_loader': { 'torch_snippets.imgaug_loader.bbs2imgaugbbs': ( 'imgaug_loader.html#bbs2imgaugbbs',
                                                                                              'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.bw': ( 'imgaug_loader.html#bw',
                                                                                   'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.crop': ( 'imgaug_loader.html#crop',
                                                                                     'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.do': ( 'imgaug_loader.html#do',
                                                                                   'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.get_size': ( 'imgaug_loader.html#get_size',
                                                                                         'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.imgaugbbs2bbs': ( 'imgaug_loader.html#imgaugbbs2bbs',
                                                                                              'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.pad': ( 'imgaug_loader.html#pad',
                                                                                    'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.rescale': ( 'imgaug_loader.html#rescale',
                                                                                        'torch_snippets/imgaug_loader.py'),
                                              'torch_snippets.imgaug_loader.rotate': ( 'imgaug_loader.html#rotate',
                                                                                       'torch_snippets/imgaug_loader.py')},
            'torch_snippets.inspector': {'torch_snippets.inspector.inspect': ('inspector.html#inspect', 'torch_snippets/inspector.py')},
            'torch_snippets.interactive_show': { 'torch_snippets.interactive_show.convert_to_nx': ( 'interactive_show.html#convert_to_nx',
                                                                                                    'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.df2graph_nodes': ( 'interactive_show.html#df2graph_nodes',
                                                                                                     'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.ishow': ( 'interactive_show.html#ishow',
                                                                                            'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.plot_graph': ( 'interactive_show.html#plot_graph',
                                                                                                 'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.plot_image': ( 'interactive_show.html#plot_image',
                                                                                                 'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.to_networkx': ( 'interactive_show.html#to_networkx',
                                                                                                  'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.tolist': ( 'interactive_show.html#tolist',
                                                                                             'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.tonp': ( 'interactive_show.html#tonp',
                                                                                           'torch_snippets/interactive_show.py'),
                                                 'torch_snippets.interactive_show.viz2': ( 'interactive_show.html#viz2',
                                                                                           'torch_snippets/interactive_show.py')},
            'torch_snippets.ipython': { 'torch_snippets.ipython.backup_all_notebooks': ( 'jupyter_notebook.html#backup_all_notebooks',
                                                                                         'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.backup_this_notebook': ( 'jupyter_notebook.html#backup_this_notebook',
                                                                                         'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.display_dfs_side_by_side': ( 'jupyter_notebook.html#display_dfs_side_by_side',
                                                                                             'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.h1': ('jupyter_notebook.html#h1', 'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.h2': ('jupyter_notebook.html#h2', 'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.h3': ('jupyter_notebook.html#h3', 'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.h4': ('jupyter_notebook.html#h4', 'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.h5': ('jupyter_notebook.html#h5', 'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.h6': ('jupyter_notebook.html#h6', 'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.is_in_notebook': ( 'jupyter_notebook.html#is_in_notebook',
                                                                                   'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.save_notebook': ( 'jupyter_notebook.html#save_notebook',
                                                                                  'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.show_big_dataframe': ( 'jupyter_notebook.html#show_big_dataframe',
                                                                                       'torch_snippets/ipython.py'),
                                        'torch_snippets.ipython.shutdown_current_notebook': ( 'jupyter_notebook.html#shutdown_current_notebook',
                                                                                              'torch_snippets/ipython.py')},
            'torch_snippets.load_defaults': { 'torch_snippets.load_defaults.exists': ( 'load_defautls.html#exists',
                                                                                       'torch_snippets/load_defaults.py'),
                                              'torch_snippets.load_defaults.loadifexists': ( 'load_defautls.html#loadifexists',
                                                                                             'torch_snippets/load_defaults.py')},
            'torch_snippets.loader': {},
            'torch_snippets.logger': { 'torch_snippets.logger.RichHandler.render': ( 'logging.html#richhandler.render',
                                                                                     'torch_snippets/logger.py'),
                                       'torch_snippets.logger.enter_exit': ('logging.html#enter_exit', 'torch_snippets/logger.py'),
                                       'torch_snippets.logger.get_console': ('logging.html#get_console', 'torch_snippets/logger.py'),
                                       'torch_snippets.logger.get_logger_level': ( 'logging.html#get_logger_level',
                                                                                   'torch_snippets/logger.py'),
                                       'torch_snippets.logger.in_logger_mode': ('logging.html#in_logger_mode', 'torch_snippets/logger.py'),
                                       'torch_snippets.logger.logger_mode': ('logging.html#logger_mode', 'torch_snippets/logger.py'),
                                       'torch_snippets.logger.notify_waiting': ('logging.html#notify_waiting', 'torch_snippets/logger.py'),
                                       'torch_snippets.logger.reset_logger': ('logging.html#reset_logger', 'torch_snippets/logger.py')},
            'torch_snippets.markup': { 'torch_snippets.markup.AttrDict': ('markups.html#attrdict', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__contains__': ( 'markups.html#attrdict.__contains__',
                                                                                        'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__delitem__': ( 'markups.html#attrdict.__delitem__',
                                                                                       'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__dir__': ( 'markups.html#attrdict.__dir__',
                                                                                   'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__eq__': ( 'markups.html#attrdict.__eq__',
                                                                                  'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__init__': ( 'markups.html#attrdict.__init__',
                                                                                    'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__iter__': ( 'markups.html#attrdict.__iter__',
                                                                                    'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__len__': ( 'markups.html#attrdict.__len__',
                                                                                   'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.__repr__': ( 'markups.html#attrdict.__repr__',
                                                                                    'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict._wrap': ('markups.html#attrdict._wrap', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.drop': ('markups.html#attrdict.drop', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.fetch': ('markups.html#attrdict.fetch', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.find_address': ( 'markups.html#attrdict.find_address',
                                                                                        'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.get': ('markups.html#attrdict.get', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.items': ('markups.html#attrdict.items', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.keys': ('markups.html#attrdict.keys', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.map': ('markups.html#attrdict.map', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.pretty': ( 'markups.html#attrdict.pretty',
                                                                                  'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.summary': ( 'markups.html#attrdict.summary',
                                                                                   'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.to_dict': ( 'markups.html#attrdict.to_dict',
                                                                                   'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.update': ( 'markups.html#attrdict.update',
                                                                                  'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.values': ( 'markups.html#attrdict.values',
                                                                                  'torch_snippets/markup.py'),
                                       'torch_snippets.markup.AttrDict.write_summary': ( 'markups.html#attrdict.write_summary',
                                                                                         'torch_snippets/markup.py'),
                                       'torch_snippets.markup._default': ('markups.html#_default', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.isnamedtupleinstance': ( 'markups.html#isnamedtupleinstance',
                                                                                       'torch_snippets/markup.py'),
                                       'torch_snippets.markup.pretty_json': ('markups.html#pretty_json', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.read_json': ('markups.html#read_json', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.read_jsonl': ('markups.html#read_jsonl', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.read_xml': ('markups.html#read_xml', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.read_yaml': ('markups.html#read_yaml', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.unpack': ('markups.html#unpack', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.write_json': ('markups.html#write_json', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.write_jsonl': ('markups.html#write_jsonl', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.write_xml': ('markups.html#write_xml', 'torch_snippets/markup.py'),
                                       'torch_snippets.markup.write_yaml': ('markups.html#write_yaml', 'torch_snippets/markup.py')},
            'torch_snippets.markup2': { 'torch_snippets.markup2.AttrDict': ('markups.html#attrdict', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__contains__': ( 'markups.html#attrdict.__contains__',
                                                                                          'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__delitem__': ( 'markups.html#attrdict.__delitem__',
                                                                                         'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__dir__': ( 'markups.html#attrdict.__dir__',
                                                                                     'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__eq__': ( 'markups.html#attrdict.__eq__',
                                                                                    'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__init__': ( 'markups.html#attrdict.__init__',
                                                                                      'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__iter__': ( 'markups.html#attrdict.__iter__',
                                                                                      'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__len__': ( 'markups.html#attrdict.__len__',
                                                                                     'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.__repr__': ( 'markups.html#attrdict.__repr__',
                                                                                      'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict._wrap': ( 'markups.html#attrdict._wrap',
                                                                                   'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.drop': ('markups.html#attrdict.drop', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.fetch': ( 'markups.html#attrdict.fetch',
                                                                                   'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.find_address': ( 'markups.html#attrdict.find_address',
                                                                                          'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.get': ('markups.html#attrdict.get', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.items': ( 'markups.html#attrdict.items',
                                                                                   'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.keys': ('markups.html#attrdict.keys', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.map': ('markups.html#attrdict.map', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.pretty': ( 'markups.html#attrdict.pretty',
                                                                                    'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.print_summary': ( 'markups.html#attrdict.print_summary',
                                                                                           'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.summary': ( 'markups.html#attrdict.summary',
                                                                                     'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.to_dict': ( 'markups.html#attrdict.to_dict',
                                                                                     'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.update': ( 'markups.html#attrdict.update',
                                                                                    'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.values': ( 'markups.html#attrdict.values',
                                                                                    'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.AttrDict.write_summary': ( 'markups.html#attrdict.write_summary',
                                                                                           'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2._default': ('markups.html#_default', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.hash_tensor': ('markups.html#hash_tensor', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.isnamedtupleinstance': ( 'markups.html#isnamedtupleinstance',
                                                                                         'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.pretty_json': ('markups.html#pretty_json', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.read_json': ('markups.html#read_json', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.read_jsonl': ('markups.html#read_jsonl', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.read_xml': ('markups.html#read_xml', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.read_yaml': ('markups.html#read_yaml', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.unpack': ('markups.html#unpack', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.write_json': ('markups.html#write_json', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.write_jsonl': ('markups.html#write_jsonl', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.write_xml': ('markups.html#write_xml', 'torch_snippets/markup2.py'),
                                        'torch_snippets.markup2.write_yaml': ('markups.html#write_yaml', 'torch_snippets/markup2.py')},
            'torch_snippets.misc': { 'torch_snippets.misc.Timer': ('misc.html#timer', 'torch_snippets/misc.py'),
                                     'torch_snippets.misc.Timer.__call__': ('misc.html#timer.__call__', 'torch_snippets/misc.py'),
                                     'torch_snippets.misc.Timer.__init__': ('misc.html#timer.__init__', 'torch_snippets/misc.py'),
                                     'torch_snippets.misc.io': ('misc.html#io', 'torch_snippets/misc.py'),
                                     'torch_snippets.misc.timeit': ('misc.html#timeit', 'torch_snippets/misc.py'),
                                     'torch_snippets.misc.track2': ('misc.html#track2', 'torch_snippets/misc.py')},
            'torch_snippets.paths': { 'torch_snippets.paths.Glob': ('paths.html#glob', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.Glob': ('paths.html#p.glob', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.cp': ('paths.html#p.cp', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.extn': ('paths.html#p.extn', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.mv': ('paths.html#p.mv', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.read_lines': ('paths.html#p.read_lines', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.rm': ('paths.html#p.rm', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.rmtree': ('paths.html#p.rmtree', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.sample': ('paths.html#p.sample', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.size': ('paths.html#p.size', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.P.write_lines': ('paths.html#p.write_lines', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.common_items': ('paths.html#common_items', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.dumpdill': ('paths.html#dumpdill', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.extn': ('paths.html#extn', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.find': ('paths.html#find', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.fname': ('paths.html#fname', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.fname2': ('paths.html#fname2', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.folder_summary': ('paths.html#folder_summary', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.input_to_str': ('paths.html#input_to_str', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.isdir': ('paths.html#isdir', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.list_zip': ('paths.html#list_zip', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.loaddill': ('paths.html#loaddill', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.makedir': ('paths.html#makedir', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.md5': ('paths.html#md5', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.output_to_path': ('paths.html#output_to_path', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.parent': ('paths.html#parent', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.readlines': ('paths.html#readlines', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.remove_duplicates': ('paths.html#remove_duplicates', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.rename_batch': ('paths.html#rename_batch', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.stem': ('paths.html#stem', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.stems': ('paths.html#stems', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.unzip_file': ('paths.html#unzip_file', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.writelines': ('paths.html#writelines', 'torch_snippets/paths.py'),
                                      'torch_snippets.paths.zip_files': ('paths.html#zip_files', 'torch_snippets/paths.py')},
            'torch_snippets.pdf_loader': { 'torch_snippets.pdf_loader.PDF': ('pdf.html#pdf', 'torch_snippets/pdf_loader.py'),
                                           'torch_snippets.pdf_loader.PDF.__getitem__': ( 'pdf.html#pdf.__getitem__',
                                                                                          'torch_snippets/pdf_loader.py'),
                                           'torch_snippets.pdf_loader.PDF.__init__': ( 'pdf.html#pdf.__init__',
                                                                                       'torch_snippets/pdf_loader.py'),
                                           'torch_snippets.pdf_loader.PDF.__len__': ( 'pdf.html#pdf.__len__',
                                                                                      'torch_snippets/pdf_loader.py'),
                                           'torch_snippets.pdf_loader.PDF.get_image': ( 'pdf.html#pdf.get_image',
                                                                                        'torch_snippets/pdf_loader.py'),
                                           'torch_snippets.pdf_loader.PDF.show': ('pdf.html#pdf.show', 'torch_snippets/pdf_loader.py')},
            'torch_snippets.registry': { 'torch_snippets.registry.parse': ('registry.html#parse', 'torch_snippets/registry.py'),
                                         'torch_snippets.registry.parse_and_resolve': ( 'registry.html#parse_and_resolve',
                                                                                        'torch_snippets/registry.py'),
                                         'torch_snippets.registry.parse_string': ( 'registry.html#parse_string',
                                                                                   'torch_snippets/registry.py'),
                                         'torch_snippets.registry.tryeval': ('registry.html#tryeval', 'torch_snippets/registry.py')},
            'torch_snippets.sklegos': { 'torch_snippets.sklegos.Cat2Num': ('sklegos.html#cat2num', 'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.Cat2Num.__init__': ( 'sklegos.html#cat2num.__init__',
                                                                                     'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.Cat2Num.fit': ('sklegos.html#cat2num.fit', 'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.Cat2Num.fit_transform': ( 'sklegos.html#cat2num.fit_transform',
                                                                                          'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.Cat2Num.transform': ( 'sklegos.html#cat2num.transform',
                                                                                      'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.GroupedEstimator.transform': ( 'sklegos.html#groupedestimator.transform',
                                                                                               'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.GroupedPredictor.transform': ( 'sklegos.html#groupedpredictor.transform',
                                                                                               'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.ImputeMissingValues': ( 'sklegos.html#imputemissingvalues',
                                                                                        'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.ImputeMissingValues.__init__': ( 'sklegos.html#imputemissingvalues.__init__',
                                                                                                 'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.ImputeMissingValues.fit': ( 'sklegos.html#imputemissingvalues.fit',
                                                                                            'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.ImputeMissingValues.fit_transform': ( 'sklegos.html#imputemissingvalues.fit_transform',
                                                                                                      'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.ImputeMissingValues.transform': ( 'sklegos.html#imputemissingvalues.transform',
                                                                                                  'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.LambdaTransformer': ( 'sklegos.html#lambdatransformer',
                                                                                      'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.LambdaTransformer.__init__': ( 'sklegos.html#lambdatransformer.__init__',
                                                                                               'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.LambdaTransformer.fit': ( 'sklegos.html#lambdatransformer.fit',
                                                                                          'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.LambdaTransformer.fit_transform': ( 'sklegos.html#lambdatransformer.fit_transform',
                                                                                                    'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.LambdaTransformer.predict': ( 'sklegos.html#lambdatransformer.predict',
                                                                                              'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.LambdaTransformer.predict_proba': ( 'sklegos.html#lambdatransformer.predict_proba',
                                                                                                    'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.LambdaTransformer.transform': ( 'sklegos.html#lambdatransformer.transform',
                                                                                                'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.MakeFrame': ('sklegos.html#makeframe', 'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.MakeFrame.__init__': ( 'sklegos.html#makeframe.__init__',
                                                                                       'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.MakeFrame.fit': ('sklegos.html#makeframe.fit', 'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.MakeFrame.predict': ( 'sklegos.html#makeframe.predict',
                                                                                      'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.MakeFrame.predict_proba': ( 'sklegos.html#makeframe.predict_proba',
                                                                                            'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.MakeFrame.transform': ( 'sklegos.html#makeframe.transform',
                                                                                        'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.SplitDateColumn': ( 'sklegos.html#splitdatecolumn',
                                                                                    'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.SplitDateColumn.__init__': ( 'sklegos.html#splitdatecolumn.__init__',
                                                                                             'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.SplitDateColumn.fit': ( 'sklegos.html#splitdatecolumn.fit',
                                                                                        'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.SplitDateColumn.predict': ( 'sklegos.html#splitdatecolumn.predict',
                                                                                            'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.SplitDateColumn.predict_proba': ( 'sklegos.html#splitdatecolumn.predict_proba',
                                                                                                  'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.SplitDateColumn.transform': ( 'sklegos.html#splitdatecolumn.transform',
                                                                                              'torch_snippets/sklegos.py'),
                                        'torch_snippets.sklegos.train_test_split': ( 'sklegos.html#train_test_split',
                                                                                     'torch_snippets/sklegos.py')},
            'torch_snippets.text_utils': {},
            'torch_snippets.thinc_parser.parser': {},
            'torch_snippets.torch_loader': {},
            'torch_snippets.trainer.capsule': { 'torch_snippets.trainer.capsule.Capsule': ( 'capsule.html#capsule',
                                                                                            'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.__init__': ( 'capsule.html#capsule.__init__',
                                                                                                     'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.after_predict': ( 'capsule.html#capsule.after_predict',
                                                                                                          'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.after_train_batch': ( 'capsule.html#capsule.after_train_batch',
                                                                                                              'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.after_validate_batch': ( 'capsule.html#capsule.after_validate_batch',
                                                                                                                 'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.before_predict': ( 'capsule.html#capsule.before_predict',
                                                                                                           'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.before_train_batch': ( 'capsule.html#capsule.before_train_batch',
                                                                                                               'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.before_validate_batch': ( 'capsule.html#capsule.before_validate_batch',
                                                                                                                  'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.evaluate': ( 'capsule.html#capsule.evaluate',
                                                                                                     'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.fit': ( 'capsule.html#capsule.fit',
                                                                                                'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.load': ( 'capsule.html#capsule.load',
                                                                                                 'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.predict_batch': ( 'capsule.html#capsule.predict_batch',
                                                                                                          'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.save': ( 'capsule.html#capsule.save',
                                                                                                 'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.train_batch': ( 'capsule.html#capsule.train_batch',
                                                                                                        'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.Capsule.validate_batch': ( 'capsule.html#capsule.validate_batch',
                                                                                                           'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.predict': ( 'capsule.html#predict',
                                                                                            'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.to': ( 'capsule.html#to',
                                                                                       'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.train': ( 'capsule.html#train',
                                                                                          'torch_snippets/trainer/capsule.py'),
                                                'torch_snippets.trainer.capsule.validate': ( 'capsule.html#validate',
                                                                                             'torch_snippets/trainer/capsule.py')},
            'torch_snippets.trainer.config': { 'torch_snippets.trainer.config.DeepLearningConfig': ( 'config.html#deeplearningconfig',
                                                                                                     'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.DeepLearningConfig.__contains__': ( 'config.html#deeplearningconfig.__contains__',
                                                                                                                  'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.DeepLearningConfig.__getitem__': ( 'config.html#deeplearningconfig.__getitem__',
                                                                                                                 'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.DeepLearningConfig.__repr__': ( 'config.html#deeplearningconfig.__repr__',
                                                                                                              'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.DeepLearningConfig.from_ini_file': ( 'config.html#deeplearningconfig.from_ini_file',
                                                                                                                   'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.DeepLearningConfig.keys': ( 'config.html#deeplearningconfig.keys',
                                                                                                          'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.GenericConfig': ( 'config.html#genericconfig',
                                                                                                'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.GenericConfig.__init__': ( 'config.html#genericconfig.__init__',
                                                                                                         'torch_snippets/trainer/config.py'),
                                               'torch_snippets.trainer.config.GenericConfig.from_ini_file': ( 'config.html#genericconfig.from_ini_file',
                                                                                                              'torch_snippets/trainer/config.py')},
            'torch_snippets.trainer.hooks': {}}}
